//Utils.js
function ParticleSystem(type, numparticles, center, gravity)
{

    this.numParticles = numparticles;
    //this.gl = null;
    this.size = 0.07        ;
    this.center = center;
    this.type = type;
    //
    this.particlePositions = new Array(); //3 * this.numparticles
    this.particleVelocities = new Array();//3 * this.numparticles
    this.particleAcceleration = new Array();//this.numparticles
    this.particleLifeTime = new Array();//this.numparticles
    this.respawnTime = new Array(); //this.numParticles (time to respawn the particle)
    this.particleStatus = new Array();
    this.particleTimer  = new Array();
    this.particleRespawnTime = new Array();
    this.textureImage = null;
    this.texture = null;
    this.useTexture = false;
    this.aliveParticles = 0;
    this.textureIndex = new Array();

    this.textureDivX = 0;
    this.textureDivY = 0;
   // this.h  = 0;

    //Particle System data

    this.gravity = gravity; //size = 3 (for all particles)
    //used to set particle initial positions
    this.center = center; //center of particle system. 3
    this.particleRespawn = false;

    

    //vbo handles
    this.particlePositionsBuffer = null; 
    this.particleStatusBuffer = null; //live or dead

    //VAO handles
    this.vao = null;
    

  

    this.genFountainParticle = function(){
        var rotInc = (Math.PI * 2) / this.numParticles;

        
        var i;
        for(i=0; i < this.numParticles; i++)
        {
            //from a circle on the ground...
            var r = rotInc * i;
            var vec = vec3.create([0,1,0]);
            var zn = vec[2] * Math.cos(r) - vec[0] * Math.sin(r);
            vec[0] = vec[2] * Math.sin(r) + vec[0] * Math.cos(r);
            vec[2] = zn;
            vec3.normalize(vec);
            this.particlePositions.push(center[0], center[1],center[2]);
            this.particleVelocities.push(vec[0], vec[1],vec[2]);
            this.particleAcceleration.push(randBetween(0.0, 2.0));
            
            this.partilcleLifeTime.push(randBetween(0.0, 2.0));
            this.particleRespawnTime.push(0);
            this.particleStatus[i] = 1;
        }
    };

    this.setGl = function(gl)
    {
        this.gl = gl;
    }

    this.genExplosionParticles = function()
    {
        
        this.useTexture = true;
        
        var i;

        for(i=0; i < this.numParticles; i++)
        {
            //from a sphere
            var vec = vec3.create([0,0,0]);
            vec[0] = randBetween(-0.5, 0.5);
            vec[1] = randBetween(-0.0, 0.5);
            vec[2] = randBetween(-0.5, 0.5);
            //vec3.normalize(vec);
            this.particlePositions.push(center[0], center[1],center[2]);
            //console.log(vec);
            this.particleVelocities.push(vec[0], vec[1],vec[2]);

            this.particleAcceleration.push(0.3);//randBetween(0.0, 1.0));
            this.textureIndex.push(i% (this.textureDivY*this.textureDivX));
            this.particleLifeTime.push(randBetween(0.0, 5.0));
            this.particleStatus.push(1);
            this.particleTimer.push(0.0);
            this.particleRespawnTime[i] = 1;
            this.particleRespawn = false;
        }
    };


    this.genSmokeParticles = function()
    {
        this.useTexture = false;
        
        var i;

        for(i=0; i < this.numParticles; i++)
        {
            //from a sphere
            var vec = vec3.create([0,0,0]);
            vec[0] = randBetween(-0.5, 0.9);
            vec[1] = randBetween(-0.5, 0.5);
            vec[2] = randBetween(-0.5, 0.5);
            //vec3.normalize(vec);
            this.particlePositions.push(center[0], center[1],center[2]);
            //console.log(vec);
            this.particleVelocities.push(vec[0], vec[1], vec[2]);
            //gl_FragColor.a = randBetween(0.25, 0.90);

            this.particleAcceleration.push(0.05);//randBetween(0.0, 1.0));
            this.textureIndex.push(i% (this.textureDivY*this.textureDivX));
            this.particleLifeTime.push(randBetween(0.0, 1.2));
            this.particleStatus.push(1);
            this.particleTimer.push(0.0);
            this.particleRespawnTime[i] = 1;
            this.particleRespawn = true;
        }
    };

    this.genFireParticles = function()
    {
        this.useTexture = true;
        
        var i;

        for(i=0; i < this.numParticles; i++)
        {
            //from a sphere
            var vec = vec3.create([0,0,0]);
            vec[0] = randBetween(-0.5, 0.5);
            vec[1] = randBetween(0.0+i/300, 0.5);
            vec[2] = randBetween(-0.5, 0.5);
            //vec3.normalize(vec);
            this.particlePositions.push(center[0], center[1],center[2]-5);
            //console.log(vec);
            this.particleVelocities.push(vec[0]+1, vec[1],vec[2]);

            this.particleAcceleration.push(0.01);//randBetween(0.0, 1.0));
            this.textureIndex.push(i% (this.textureDivY*this.textureDivX));
            this.particleLifeTime.push(randBetween(0.0, 2.0));
            this.particleStatus.push(1);
            this.particleTimer.push(0.0);
            this.particleRespawnTime[i] = 1;
            this.particleRespawn = true;
        }
    };
    

    this.updateParticleSystem = function(timeElapsed, gl)
    {
        var i;

        this.aliveParticles = 0;
        
        for(i = 0; i < this.numParticles; i++)
        {
            
            this.particleTimer[i] += timeElapsed;

            if(this.particleStatus[i] == 1)
            {
                this.aliveParticles++;
            
                this.particlePositions[i * 3  ] += this.particleVelocities[i * 3]    * timeElapsed + ((1/2) * this.particleAcceleration[i] * (timeElapsed * timeElapsed))  ;
                this.particlePositions[i * 3+1] += this.particleVelocities[i * 3+1]  * timeElapsed + ((1/2) * this.particleAcceleration[i] * (timeElapsed * timeElapsed))  ;
                this.particlePositions[i * 3+2] += this.particleVelocities[i * 3+2]  * timeElapsed + ((1/2) * this.particleAcceleration[i] * (timeElapsed * timeElapsed))  ;
                if(this.particleTimer[i] > this.particleLifeTime[i])//'morre'
                {
                    this.particleStatus[i] = 0;
                    this.particleTimer[i] = 0;
                    this.aliveParticles--;
                }

            }
            else if(this.particleStatus[i] == 0 && this.particleRespawn)
            {
                if(this.particleTimer[i] >= this.particleRespawnTime[i])
                {
                    this.aliveParticles++;
                    this.particleStatus[i] = 1;
                    this.particleTimer[i] = 0;
                    this.particlePositions[i * 3  ] = this.center[0] + randBetween(-0.10, 0.10);
                    this.particlePositions[i * 3+1] = this.center[1] + randBetween(-0.15, 0.15);
                    this.particlePositions[i * 3+2] = this.center[2] + randBetween(-0.15, 0.15);
                }                  
            }            
        }
        ///console.log(this.particlePositions);
        //this.updateBuffers(gl);
    };

    //create GPU buffers
    this.createBuffers = function(gl)
    {
    
        //buffer for particle positions
        this.particlePositionsBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.particlePositionsBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(this.particlePositions), gl.DYNAMIC_DRAW);

        this.particleStatusBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.particleStatusBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Int32Array(this.particleStatus), gl.DYNAMIC_DRAW);
        

        this.textureCoordBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.textureCoordBuffer);
        
        
    };
    //
    this.updateBuffers = function(gl)
    {
        var texCoord = new Array();
        var b = new Array();
        var tamTex = 1.0 / this.textureDivX;
        for(i = 0; i < this.particlePositions.length; i++){
            //qudrado para cada partícula
            
            if(this.particleStatus[i] == 1)
            {

                var idx = this.textureIndex[i];//indice na textura
                var l = Math.floor(idx / this.textureDivX); //linha e colluna na textura
                var c = idx % this.textureDivX; 
                //console.log("Index: "+idx);    
                //(0,1)
                b.push(this.particlePositions[i * 3]-this.size, this.particlePositions[i*3+1]-this.size, this.particlePositions[i*3+2]);
                texCoord.push(c *tamTex, l* tamTex);//faz o mapeamento na subimagem da textura.
                //texCoord.push(0,0);


                //(1,1)
                b.push(this.particlePositions[i * 3]+this.size, this.particlePositions[i*3+1]-this.size, this.particlePositions[i*3+2]);
                texCoord.push(c *tamTex, l* tamTex + tamTex);
                //texCoord.push(1,0);


                //(1,0)
                b.push(this.particlePositions[i * 3]+this.size, this.particlePositions[i*3+1]+this.size, this.particlePositions[i*3+2]);
                texCoord.push( c *tamTex+tamTex, l* tamTex + tamTex);
                //texCoord.push(1,1);
   


                //(1,1)
                b.push(this.particlePositions[i * 3]+this.size, this.particlePositions[i*3+1]+this.size, this.particlePositions[i*3+2]);
                texCoord.push(c *tamTex+tamTex, l* tamTex + tamTex );
                //texCoord.push(1,1);

                //(0,1)
                b.push(this.particlePositions[i * 3]-this.size, this.particlePositions[i*3+1]+this.size, this.particlePositions[i*3+2]);
                texCoord.push( c *tamTex+tamTex, l* tamTex);// -tamTex);
                //texCoord.push(0,1);
                
                //(0,1)
                b.push(this.particlePositions[i * 3]-this.size, this.particlePositions[i*3+1]-this.size, this.particlePositions[i*3+2]);
                texCoord.push( c *tamTex, l* tamTex);
                //texCoord.push(0,0);
            }
        }
         

        this.particlePositionsBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.particlePositionsBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(b), gl.DYNAMIC_DRAW);

        
/*        gl.bindBuffer(gl.ARRAY_BUFFER, this.particleStatusBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Int32Array(texCoord), gl.DYNAMIC_DRAW);
  */      

        this.textureCoordBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.textureCoordBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(texCoord), gl.DYNAMIC_DRAW);

    };

    this.draw = function(gl){
        //console.log(this.h);
        //this.updateBuffers(gl);

        var particlePositionAttrib = gl.getAttribLocation(glProgram, "particlePosition");
        gl.enableVertexAttribArray(particlePositionAttrib);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.particlePositionsBuffer);
        gl.vertexAttribPointer(particlePositionAttrib, 3, gl.FLOAT, false, 0, 0);


        var textureCoordAttrib = gl.getAttribLocation(glProgram, "textureCoord");
        gl.enableVertexAttribArray(textureCoordAttrib);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.textureCoordBuffer);
        gl.vertexAttribPointer(textureCoordAttrib, 2, gl.FLOAT, false, 0, 0);


       /* this.particleStatusBuffer = gl.getAttribLocation(glProgram, "aVertexColor");
        gl.enableVertexAttribArray(this.particleStatusBuffer);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.particleStatus );
        gl.vertexAttribPointer(this.particleStatusBuffer, 1, gl.INT, false, 0, 0);
*/
        if(this.aliveParticles>0)
            gl.drawArrays(gl.TRIANGLES, 0, this.aliveParticles*6);
    }

    this.setupTexture=function(gl)
    {
        
        this.texture = gl.createTexture();
        gl.bindTexture(gl.TEXTURE_2D, this.texture);
        gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, this.textureImage);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        if( !gl.isTexture(this.texture) ){
            console.log("Error: Texture is invalid");
        }
        var samplerUniform = gl.getUniformLocation(glProgram, "uSampler");
        gl.uniform1i(samplerUniform, 0);

    }

    this.loadTexture=function(image, gl)
    {
        this.textureImage = image;
    //    this.textureImage.onload=function(){
      //      ParticleSystem.this.setupTexture(gl);
     //   };
        //this.textureImage.src = texSrc;
        this.setupTexture(gl);
        
    } 

/******************************************************************/
    if(this.type == 'smoke')
    {
        this.textureDivX = 4;// 4;
        this.textureDivY = 4;//4;
        this.genSmokeParticles();
        
    }
    else if( this.type == 'fountain')
    {

        this.genFountainParticle();
    }
    else if(this.type == "fire")
    {
        this.textureDivX = 4;// 4;
        this.textureDivY = 4;//4;
        this.genFireParticles();
    }
    else if(this.type == "explosion")
    {
        this.textureDivX = 4;// 4;
        this.textureDivY = 4;//4;
        this.genExplosionParticles();
        
    }
}


