//https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext

///configure scene
var gl = null; //OpenGL context

var canvas = null; // the canvas

//canvas bkg / clear color
var bkgColor = null;

//OpenGL shader program
var glProgram = null;

//proj matrix
var pMatrix = mat4.create();
//model matrix
var mMatrix = mat4.create();

//view matrix
var vMatrix = mat4.create();

var context = "webgl";

var numFps = 0;

var partSystem = new ParticleSystem('smoke', 9000,[0.0,-0.0,8.0],[0.0,0.0,0.0]);

var useTexture = 1;
function configure()
{
   
    

    bkgColor = new Color(0,0,0,1.0);

    var startTime = new Date();
    var endTime   = null;
     
    canvas = document.getElementById("webglcontainer");
    
    mat4.identity(pMatrix);
    //mat4.ortho(-3, 3, -3, 3,  -0.1, 20, pMatrix);
    mat4.perspective( 45, canvas.width / canvas.height, 0.01, 100.0,pMatrix);
    
    mat4.identity(vMatrix);
    mat4.lookAt([0,0,10], [0,0,0],[0,1,0],vMatrix);
    //mat4.rotate(vMatrix,90,[0,1,0],vMatrix);
    
    try{
        gl = canvas.getContext(context);
    }catch(e){
    }
    if(!gl)
    {
        throw new Error("Error: Your browser does not appear to support WebGL.");
    }
    initShaders(document.getElementById("shader-vs").innerHTML, document.getElementById("shader-fs").innerHTML);
    partSystem.createBuffers(gl);
    partSystem.loadTexture(document.getElementById("smokeTexture"),gl);    


    gl.enable(gl.DEPTH_TEST);
    

    


   //loop de animacao. Ele repete automaticamente.
    (function animLoop(){
        
        draw();
        numFps++;
        endTime = new Date();
        if(endTime - startTime >= 1000)
        {
            startTime = endTime;
            //console.log("FPS: "+numFps);
            numFps = 0;
        }
        window.requestAnimationFrame(animLoop, canvas);
    })();


}

function draw()
{
    
    gl.enable(gl.BLEND);
    gl.blendEquation( gl.FUNC_ADD );
    gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
    gl.clearColor(bkgColor.r, bkgColor.g,   bkgColor.b, bkgColor.a);
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);// | gl.DEPTH_BUFFER_BIT);
    //gl.enable(gl.DEPTH_TEST);
    gl.viewport(0, 0, canvas.width, canvas.height);
    
    partSystem.updateParticleSystem(1.0/60, gl);


    partSystem.updateBuffers(gl);

    var matUni = gl.getUniformLocation(glProgram, "uMVMatrix");
    gl.uniformMatrix4fv(matUni, false, vMatrix);

    var matPro = gl.getUniformLocation(glProgram, "uPMatrix");
    gl.uniformMatrix4fv(matPro, false, pMatrix);

    var useTex = gl.getUniformLocation(glProgram, "useTexture");
    gl.uniform1i(useTex, useTexture);
    


    partSystem.draw(gl);
    gl.disable(gl.BLEND);
    
}


function Color (r, g, b, a){
    this.r = r;
    this.g = g;
    this.b = b;
    this.a = a;
}

function initShaders(vertex, fragment){
    if( (vertex !=null) && (fragment !=null)){
        //compile shaders
        vertexShader = makeShader(vertex, gl.VERTEX_SHADER,"vertex");
        fragmentShader = makeShader(fragment, gl.FRAGMENT_SHADER,"Fragment");
        //create program
        glProgram = gl.createProgram();
        //attach and link shaders to the program
        gl.attachShader(glProgram, vertexShader);
        gl.attachShader(glProgram, fragmentShader);
        gl.linkProgram(glProgram);
        if (!gl.getProgramParameter(glProgram, gl.LINK_STATUS)) {
        alert("Unable to initialize the shader program.");
        }
        //use program
        gl.useProgram(glProgram);
    }
}
function makeShader(src, type, what){
    //compile the vertex shader
    var shader = gl.createShader(type);
    gl.shaderSource(shader, src);
    gl.compileShader(shader);
    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
        alert("Error compiling shader "+what+": "+" "+ gl.getShaderInfoLog(shader));
    }
    return shader;
}
